import { lstatSync, readdirSync } from 'fs';
import { join, basename, resolve, dirname } from 'path';
function lstatSafe(pathName) {
    try {
        const result = lstatSync(pathName);
        result.name = basename(resolve(pathName));
        return result;
    }
    catch (err) {
        return err;
    }
}
function readdirSafe(pathName) {
    try {
        return readdirSync(pathName, { withFileTypes: true });
    }
    catch (err) {
        return [err];
    }
}
function walkDeep(pathName, walkFunc, dirent) {
    // If the callback returns false, the Dirent errored, or entry is a file, we can skip. Otherwise, walk our directory.
    if (walkFunc(dirname(pathName), dirent) === false || dirent instanceof Error || !dirent.isDirectory()) {
        return;
    }
    for (const entry of readdirSafe(pathName)) {
        walkDeep(join(pathName, entry.name), walkFunc, entry);
    }
}
// Get the very first file or folder an walk deep.
const walk = (pathName, walkFunc) => walkDeep(pathName, walkFunc, lstatSafe(pathName));
export default walk;
//# sourceMappingURL=walk.js.map